@@ -4,9 +4,10 @@ use 5.008_001;
 use strict;
 use warnings;
 
-our $VERSION = '1.0000';
+our $VERSION = '1.0099_01';
 
 use Carp              ();
+use Fcntl             ();
 use File::Spec        ();
 use Exporter          ();
 use Data::MessagePack ();
@@ -28,7 +29,7 @@ our @EXPORT_OK = qw(
     html_builder
 );
 
-my $BYTECODE_VERSION = '1.2';
+my $BYTECODE_VERSION = '1.3';
 
 # $bytecode_version + $fullpath + $compiler_and_parser_options
 my $XSLATE_MAGIC   = qq{xslate;$BYTECODE_VERSION;%s;%s;};
@@ -60,6 +61,7 @@ package Text::Xslate::Engine;
 use Text::Xslate::Util qw(
     import_from
     make_error
+    dump
 );
 
 BEGIN {
@@ -101,10 +103,10 @@ my %builtin = (
     mark_raw     => \&Text::Xslate::Util::mark_raw,
     unmark_raw   => \&Text::Xslate::Util::unmark_raw,
     uri          => \&Text::Xslate::Util::uri_escape,
-    is_array_ref => \&_builtin_is_array_ref,
-    is_hash_ref  => \&_builtin_is_hash_ref,
+    is_array_ref => \&Text::Xslate::Util::is_array_ref,
+    is_hash_ref  => \&Text::Xslate::Util::is_hash_ref,
 
-    dump       => \&Text::Xslate::Util::p,
+    dump         => \&Text::Xslate::Util::dump,
 );
 
 sub default_functions { +{} } # overridable
@@ -294,6 +296,7 @@ sub slurp {
 
     open my($source), '<' . $self->{input_layer}, $fullpath
         or $self->_error("LoadError: Cannot open $fullpath for reading: $!");
+    flock($source, Fcntl::LOCK_SH());
     local $/;
     return scalar <$source>;
 }
@@ -325,7 +328,10 @@ sub _load_source {
                 or Carp::carp("Xslate: Cannot make directory $cachepath (ignored): $@");
         }
 
-        if(open my($out), '>:raw', $cachepath) {
+        if(sysopen my($out), $cachepath, Fcntl::O_WRONLY() | Fcntl::O_CREAT()) {
+            flock $out, Fcntl::LOCK_EX();
+            truncate $out, 0 or Carp::croak("Xslate: failed to truncate: $!");
+            binmode($out);
             $self->_save_compiled($out, $asm, $fullpath, utf8::is_utf8($source));
 
             if(!close $out) {
@@ -372,6 +378,7 @@ sub _load_compiled {
     my $cachepath = $fi->{cachepath};
     open my($in), '<:raw', $cachepath
         or $self->_error("LoadError: Cannot open $cachepath for reading: $!");
+    flock($in, Fcntl::LOCK_SH());
 
     my $magic = $self->_magic_token($fi->{fullpath});
     my $data;
@@ -441,6 +448,7 @@ sub _magic_token {
         ref($self->{compiler}) || $self->{compiler},
         $self->_extract_options(\%parser_option),
         $self->_extract_options(\%compiler_option),
+        [sort keys %{ $self->{function} } ],
     ]);
 
     if(ref $fullpath) { # ref to content string
@@ -495,21 +503,10 @@ sub compile {
     return $self->_compiler->compile(@_, from_include => $self->{from_include});
 }
 
-sub _builtin_is_array_ref {
-    return ref($_[0]) eq 'ARRAY';
-}
-
-sub _builtin_is_hash_ref {
-    return ref($_[0]) eq 'HASH';
-}
-
 sub _error {
     die make_error(@_);
 }
 
-sub dump :method {
-    goto &Text::Xslate::Util::p;
-}
 
 package Text::Xslate;
 1;
@@ -521,18 +518,13 @@ Text::Xslate - Scalable template engine for Perl5
 
 =head1 VERSION
 
-This document describes Text::Xslate version 1.0000.
+This document describes Text::Xslate version 1.0099_01.
 
 =head1 SYNOPSIS
 
-    use Text::Xslate;
+    use Text::Xslate qw(mark_raw);
 
-    my $tx = Text::Xslate->new(
-        # the following options are optional.
-        path       => ['.'],
-        cache_dir  => "$ENV{HOME}/.xslate_cache",
-        cache      => 1,
-    );
+    my $tx = Text::Xslate->new();
 
     my %vars = (
         title => 'A list of books',
@@ -541,6 +533,9 @@ This document describes Text::Xslate version 1.0000.
             { title => 'Programming Perl'      },
             # ...
         ],
+
+        # mark HTML components as raw not to escape its HTML tags
+        gadget => mark_raw('<div class="gadget">...</div>'),
     );
 
     # for files
@@ -568,6 +563,8 @@ and Template-Toolkit 2, but the central philosophy of Xslate is different
 from them. That is, the philosophy is B<sandboxing> that the template logic
 should not have no access outside the template beyond your permission.
 
+Other remarkable features are as follows:
+
 =head2 Features
 
 =head3 High performance
@@ -578,7 +575,7 @@ which is highly optimized for rendering templates. Thus, Xslate is
 much faster than any other template engines.
 
 Here is a result of F<benchmark/x-rich-env.pl> to compare various template
-engines in "rich" environment where applications are persistent and XS modules
+engines in I<rich> environment where applications are persistent and XS modules
 are available.
 
     $ perl -Mblib benchmark/x-rich-env.pl
@@ -610,10 +607,10 @@ Text::ClearSilver and HTML::Template::Pro.
 
 There are benchmark scripts in the F<benchmark/> directory.
 
-=head3 Auto escaping to HTML meta characters
+=head3 Smart escaping for HTML meta characters
 
 All the HTML meta characters in template expressions the engine interpolates
-into template texts are automatically escaped, so the output has no
+into template texts are escaped automatically, so the output has no
 possibility to XSS by default.
 
 =head3 Template cascading
@@ -657,21 +654,23 @@ checks the freshness of the original templates every time.
 If I<$level> E<gt>= 2, caches will be created but the freshness
 will not be checked.
 
-I<$level> == 0 uses no caches. It's provided for testing.
+I<$level> == 0 uses no caches, which is provided for testing.
 
 =item C<< cache_dir => $dir // "$ENV{HOME}/.xslate_cache" >>
 
 Specifies the directory used for caches. If C<$ENV{HOME}> doesn't exist,
 C<< File::Spec->tmpdir >> will be used.
 
-You B<should> specify this option on productions.
+You B<should> specify this option for productions to avoid conflicts of
+template names.
 
 =item C<< function => \%functions >>
 
 Specifies a function map which contains name-coderef pairs.
 A function C<f> may be called as C<f($arg)> or C<$arg | f> in templates.
 
-There are builtin filters which are not overridable.
+There are built-in functions which are not overridable.
+See L<Text::Xslate::Manual::Builtin> for details.
 
 =item C<< module => [$module => ?\@import_args, ...] >>
 
@@ -722,6 +721,9 @@ If C<< $level >= 2 >>, all the possible errors will be warned.
 Specify the template suffix, which is used for C<cascade> and C<include>
 in Kolon.
 
+Note that this is used for static name resolution. That is, the compiler
+uses it but the runtime engine doesn't.
+
 =item C<< syntax => $name // 'Kolon' >>
 
 Specifies the template syntax you want to use.
@@ -734,8 +736,10 @@ This option is passed to the compiler directly.
 =item C<< type => $type // 'html' >>
 
 Specifies the output content type. If I<$type> is C<html> or C<xml>,
-template expressions are interpolated via the C<html-escape> filter.
-If I<$type> is C<text>, template expressions are interpolated as they are.
+smart escaping is applied to template expressions. That is,
+they are interpolated via the C<html_escape> filter.
+If I<$type> is C<text> smart escaping is not applied so that it is
+suitable for plain texts like e-mails.
 
 I<$type> may be B<html>, B<xml> (identical to C<html>), and B<text>.
 
@@ -743,8 +747,7 @@ This option is passed to the compiler directly.
 
 =item C<< line_start => $token // $parser_defined_str >>
 
-Specify the token to start line code as a string, which C<quotemeta> will be applied to. If you give C<undef> to this option, the line code style will be
-disabled.
+Specify the token to start line code as a string, which C<quotemeta> will be applied to. If you give C<undef>, the line code style is disabled.
 
 This option is passed to the parser via the compiler.
 
@@ -772,27 +775,47 @@ Specify the footer template files, which are inserted to the foot of each templa
 
 This option is passed to the compiler.
 
+=item C<< warn_handler => \&cb >>
+
+Specify the callback I<&cb> which is called on warnings.
+
+This option is experimental.
+
+=item C<< die_handler => \&cb >>
+
+Specify the callback I<&cb> which is called on fatal errors.
+
+This option is experimental.
+
 =back
 
 =head3 B<< $tx->render($file, \%vars) :Str >>
 
-Renders a template file with variables, and returns the result.
+Renders a template file with given variables, and returns the result.
 I<\%vars> is optional.
 
 Note that I<$file> may be cached according to the cache level.
 
 =head3 B<< $tx->render_string($string, \%vars) :Str >>
 
-Renders a template string with variables, and returns the result.
+Renders a template string with given variables, and returns the result.
 I<\%vars> is optional.
 
-Note that I<$string> is never cached, so this method may not be suitable for
-productions.
+Note that I<$string> is never cached, so this method should be avoided in
+production environment. If you want in-memory templates, consider the I<path>
+option for HASH references which are cached as you expect:
+
+    my %vpath = (
+        'hello.tx' => 'Hello, <: $lang :> world!',
+    );
+
+    my $tx = Text::Xslate->new( path => \%vpath );
+    print $tx->render('hello.tx', { lang => 'Xslate' });
 
 =head3 B<< $tx->load_file($file) :Void >>
 
-Loads I<$file> into memory for following C<render($file, \%vars)>.
-Compiles and saves it as caches if needed.
+Loads I<$file> into memory for following C<render()>.
+Compiles and saves it as disk caches if needed.
 
 It is a good idea to load templates before applications fork.
 Here is an example to to load all the templates which is in a given path:
@@ -829,6 +852,10 @@ This method is significant when it is called by template functions and methods.
 Returns the current line number while executing. Otherwise returns C<undef>.
 This method is significant when it is called by template functions and methods.
 
+=head3 B<< Text::Xslate->print(...) :Void >>
+
+Adds the argument into the output buffer. This method is available on executing.
+
 =head2 Exportable functions
 
 =head3 C<< mark_raw($str :Str) :RawStr >>
@@ -847,7 +874,7 @@ For example:
     # => Mailaddress: Foo &lt;foo@example.com&gt;
 
 This function is available in templates as the C<mark_raw> filter, although
-the use of it is discouraged.
+the use of it is strongly discouraged.
 
 =head3 C<< unmark_raw($str :Str) :Str >>
 
@@ -901,7 +928,7 @@ The C<xslate(1)> command is provided as a CLI to the Text::Xslate module,
 which is used to process directory trees or to evaluate one liners.
 For example:
 
-    $ xslate -D name=value -o dest_path src_path
+    $ xslate -Dname=value -o dest_path src_path
 
     $ xslate -e 'Hello, <: $ARGV[0] :> wolrd!' Xslate
     $ xslate -s TTerse -e 'Hello, [% ARGV.0 %] world!' TTerse
@@ -985,7 +1012,7 @@ Augment modifiers.
 
 =item *
 
-Loop controls.
+Loop controls (last / next).
 
 =item *
 
@@ -995,6 +1022,16 @@ Default arguments and named arguments for macros.
 
 =cut
 
+=head1 SUPPORT
+
+WEB: L<http://xslate.org/>
+
+IRC: #xslate @ irc.perl.org
+
+REPOSITORY:
+    http://github.com/gfx/p5-Text-Xslate/
+    git://github.com/gfx/p5-Text-Xslate.git
+
 =head1 BUGS
 
 All complex software has bugs lurking in it, and this module is no
@@ -1019,12 +1056,6 @@ Xslate command:
 
 L<xslate>
 
-The Xslate web site and public repository:
-
-L<http://xslate.org/>
-
-L<http://github.com/gfx/p5-Text-Xslate>
-
 Other template modules that Xslate has been influenced by:
 
 L<Text::MicroTemplate>
@@ -1049,6 +1080,10 @@ L<Template::Benchmark>
 
 L<http://xslate.org/benchmark.html>
 
+Papers:
+
+L<http://www.cs.usfca.edu/~parrt/papers/mvc.templates.pdf> -  Enforcing Strict Model-View Separation in Template Engines
+
 =head1 ACKNOWLEDGEMENT
 
 Thanks to lestrrat for the suggestion to the interface of C<render()>,
@@ -1059,10 +1094,10 @@ Thanks to tokuhirom for the ideas, feature requests, encouragement, and bug find
 
 Thanks to gardejo for the proposal to the name B<template cascading>.
 
-Thanks to jjn1056 to the concept of template overlay (now implemented as C<cascade with ...>).
-
 Thanks to makamaka for the contribution of Text::Xslate::PP.
 
+Thanks to jjn1056 to the concept of template overlay (now implemented as C<cascade with ...>).
+
 Thanks to typester for the various inspirations.
 
 Thanks to clouder for the patch of adding C<AND> and C<OR> to TTerse.
@@ -1079,6 +1114,16 @@ Thanks to Mons Anderson for the bug reports and patches.
 
 Thanks to hirose31 for the feature requests and bug reports.
 
+Thanks to c9s for the contribution of the documents.
+
+Thanks to shiba_yu36 for the bug reports.
+
+Thanks to kane46taka for the bug reports.
+
+Thanks to cho45 for the bug reports.
+
+Thanks to shmorimo for the bug reports.
+
 =head1 AUTHOR
 
 Fuji, Goro (gfx) E<lt>gfuji(at)cpan.orgE<gt>
