@@ -21,6 +21,7 @@ use constant {
     _DUMP_ASM => scalar($DEBUG =~ /\b dump=asm \b/xms),
     _DUMP_AST => scalar($DEBUG =~ /\b dump=ast \b/xms),
     _DUMP_GEN => scalar($DEBUG =~ /\b dump=gen \b/xms),
+    _DUMP_CAS => scalar($DEBUG =~ /\b dump=cascade \b/xms),
 
     _OP_NAME    => 0,
     _OP_ARG     => 1,
@@ -367,6 +368,7 @@ sub compile_ast {
 
 sub _process_cascade {
     my($self, $cascade, $args, $main_code) = @_;
+    printf STDERR "cascade %s\n", $self->file, $cascade->dump if _DUMP_CAS;
     my $engine = $self->engine
         || $self->_error("Cannot cascade templates without Xslate engine", $cascade);
 
@@ -456,6 +458,7 @@ sub _process_cascade {
 
 sub _process_cascade_file {
     my($self, $file, $base_code) = @_;
+    printf STDERR "cascade file %s\n", Mouse::Util::dump($file) if _DUMP_CAS;
     my $mtable = $self->macro_table;
 
     for(my $i = 0; $i < @{$base_code}; $i++) {
@@ -629,6 +632,7 @@ sub _generate_include {
             : $self->compile_ast($file) ),
         $self->opcode( $node->id => undef, line => $node->line ),
     );
+
     if(defined(my $vars = $node->second)) {
         @code = ($self->opcode('enter'),
             $self->_localize_vars($vars),
@@ -713,6 +717,26 @@ sub _generate_for {
     return @code;
 }
 
+sub _generate_for_else {
+    my($self, $node) = @_;
+
+    my $for_block  = $node->first;
+    my $else_block = $node->second;
+
+    my @code = (
+        $self->compile_ast($for_block),
+    );
+
+    # 'for' block sets __a with true if the loop count > 0
+    my @else = $self->compile_ast($else_block);
+    push @code, (
+        $self->opcode( or => scalar(@else) + 1, comment => 'for-else' ),
+        @else,
+    );
+
+    return @code;
+}
+
 sub _generate_while {
     my($self, $node) = @_;
     my $expr  = $node->first;
@@ -1200,6 +1224,17 @@ sub _localize_vars {
     my($self, $vars) = @_;
     my @localize;
     my @pairs = @{$vars};
+
+    if( (@pairs % 2) != 0 ) {
+        if(@pairs == 1) {
+            return $self->compile_ast(@pairs),
+                $self->opcode( 'localize_vars' );
+        }
+        else {
+            $self->_error("You must pass pairs of expressions to include");
+        }
+    }
+
     while(my($key, $expr) = splice @pairs, 0, 2) {
         if(!any_in($key->arity, qw(literal variable))) {
             $self->_error("You must pass a simple name to localize variables", $key);
