@@ -3,8 +3,7 @@ package Text::Xslate::PP;
 use 5.008_001;
 use strict;
 
-our $VERSION = '1.0000';
-$VERSION =~ s/_//; # for developpers versions
+our $VERSION = '1.0099_01';
 
 BEGIN{
     $ENV{XSLATE} = ($ENV{XSLATE} || '') . '[pp]';
@@ -18,7 +17,7 @@ use constant _PP_OPCODE  => scalar($DEBUG =~ /\b pp=opcode  \b/xms);
 use constant _PP_BOOSTER => scalar($DEBUG =~ /\b pp=booster \b/xms);
 use constant _PP_BACKEND =>   _PP_OPCODE  ? 'Opcode'
                             : _PP_BOOSTER ? 'Booster'
-                            :               'Booster'; # default
+                            :               'Opcode'; # default
 use constant _PP_ERROR_VERBOSE => scalar($DEBUG =~ /\b pp=verbose \b/xms);
 
 use constant _DUMP_LOAD => scalar($DEBUG =~ /\b dump=load \b/xms);
@@ -28,11 +27,18 @@ use Text::Xslate::PP::State;
 use Text::Xslate::PP::Type::Raw;
 use Text::Xslate ();
 
-use Carp ();
+use Scalar::Util ();
+use overload     ();
+use Carp         ();
 
 require sprintf('Text/Xslate/PP/%s.pm', _PP_BACKEND);
 
 my $state_class = 'Text::Xslate::PP::' . _PP_BACKEND;
+$VERSION eq $state_class->VERSION
+    or die "Version mismatch (" . __PACKAGE__ . "/$VERSION v.s. "
+         . "$state_class/" . $state_class->VERSION . ")";
+
+$VERSION =~ s/_//; # for developpers versions
 
 if(_PP_ERROR_VERBOSE) {
     Carp->import('verbose');
@@ -62,7 +68,7 @@ our %html_escape = (
     '<' => '&lt;',
     '>' => '&gt;',
     '"' => '&quot;',
-    "'" => '&apos;',
+    "'" => '&#39;', # IE8 doesn't support &apos; in title
 );
 our $html_metachars = sprintf '[%s]', join '', map { quotemeta } keys %html_escape;
 
@@ -129,6 +135,19 @@ sub current_line {
         : undef;
 }
 
+sub print {
+    shift;
+    if(defined $_current_st) {
+        foreach my $s(@_) {
+            $_current_st->print($s);
+        }
+    }
+    else {
+        Carp::croak('You cannot call print() method outside render()');
+    }
+    return '';
+}
+
 # >> copied and modified from Text::Xslate
 
 sub _assemble {
@@ -175,9 +194,7 @@ sub _assemble {
 
     my $len = scalar( @$asm );
 
-    my $mainframe = tx_push_frame( $st );
-    $mainframe->[ Text::Xslate::PP::TXframe_NAME ]    = 'main';
-    $mainframe->[ Text::Xslate::PP::TXframe_RETADDR ] = $len;
+    $st->push_frame('main', $len);
 
     $st->code_len( $len );
 
@@ -341,12 +358,43 @@ sub _assemble {
         $s =~ s/($uri_unsafe_rfc3986)/sprintf '%%' . '%02X', ord $1/xmsgeo;
         return $s;
     }
+
+    sub is_array_ref { ref($_[0]) eq 'ARRAY' }
+    sub is_hash_ref  { ref($_[0]) eq 'HASH'  }
 }
 
 #
 # INTERNAL
 #
 
+sub tx_check_itr_ar {
+    my ( $st, $ar, $frame, $line ) = @_;
+    return $ar if ref($ar) eq 'ARRAY';
+
+    if ( defined $ar ) {
+        if(my $x = Text::Xslate::PP::sv_is_ref($ar, 'ARRAY', '@{}')) {
+            return $x;
+        }
+
+        $st->error( [$frame, $line],
+            "Iterator variables must be an ARRAY reference, not %s",
+            Text::Xslate::Util::neat( $ar ) );
+    }
+    return [];
+}
+
+sub sv_is_ref {
+    my($sv, $t, $ov) = @_;
+    return $sv if ref($sv) eq $t;
+
+    if(Scalar::Util::blessed($sv)
+        && (my $m = overload::Method($sv, $ov))) {
+        $sv = $sv->$m(undef, undef);
+        return $sv if ref($sv) eq $t;
+    }
+    return undef;
+}
+
 sub tx_sv_eq {
     my($x, $y) = @_;
     if ( defined $x ) {
@@ -414,7 +462,6 @@ sub tx_repeat {
     elsif(!Scalar::Util::looks_like_number($rhs)) {
         $_current_st->error(undef, "Repeat count must be a number, not %s",
             Text::Xslate::Util::neat($rhs));
-        return undef;
     }
     else {
         if( ref( $lhs ) eq TXt_RAW ) {
@@ -424,16 +471,7 @@ sub tx_repeat {
             return $lhs x $rhs;
         }
     }
-}
-
-sub tx_push_frame {
-    my ( $st ) = @_;
-
-    if ( $st->current_frame > 100 ) {
-        Carp::croak("Macro call is too deep (> 100)");
-    }
-
-    return $st->frame->[ $st->current_frame( $st->current_frame + 1 ) ] ||= [];
+    return '';
 }
 
 
@@ -520,7 +558,7 @@ sub tx_all_deps_are_fresh {
     return 1;
 }
 
-sub tx_execute { 
+sub tx_execute {
     my ( $st, $vars ) = @_;
     no warnings 'recursion';
 
@@ -528,8 +566,8 @@ sub tx_execute {
         Carp::croak("Execution is too deep (> 100)");
     }
 
-    $st->{pc}   = 0;
-    $st->{vars} = $vars;
+    local $st->{pc}   = 0;
+    local $st->{vars} = $vars;
 
     local $_depth      = $_depth + 1;
     local $_current_st = $st;
@@ -552,7 +590,7 @@ sub tx_execute {
 
         local $st->{sa};
         local $st->{sb};
-        $st->{output} = '';
+        local $st->{output} = '';
         $st->{code}->[0]->{ exec_code }->( $st );
         return $st->{output};
     }
@@ -629,11 +667,11 @@ Text::Xslate::PP - Yet another Text::Xslate runtime in pure Perl
 
 =head1 VERSION
 
-This document describes Text::Xslate::PP version 1.0000.
+This document describes Text::Xslate::PP version 1.0099_01.
 
 =head1 DESCRIPTION
 
-This module implements Text::Xslate runtime engine in pure Perl.
+This module implements a Text::Xslate runtime engine in pure Perl.
 Normally it will be loaded if it fails to load XS. So you do not need
 to use this module explicitly.
 
@@ -648,58 +686,14 @@ If you want to use Text::Xslate::PP, however, you can use it.
 
 XS/PP mode might be switched with C<< $ENV{XSLATE} = 'pp' or 'xs' >>.
 
-From 0.1024 on, there are two pure Perl engines.
-C<Text::Xslate::PP::Booster>, enabled by C<< $ENV{XSLATE} = 'pp=booster' >>,
-generates optimized Perl code from intermediate code.
-C<Text::Xslate::PP::Opcode>, enabled by C<< $ENV{XSLATE} = 'pp=opcode' >>,
-executes intermediate code directly, emulating the virtual machine in pure Perl.
-
-PP::Booster is much faster than PP::Opcode, but it may be less stable.
-The default pure Perl engine is B<PP::Booster>, so if you run into problems,
-please try C<< $ENV{XSLATE} = 'pp=opcode' >>.
-
 C<< $ENV{XSLATE} = 'pp=verbose' } >> may be useful for debugging.
 
-=head1 NOTE
-
-=head2 Performance
-
-There might be cases where you cannot use XS modules.
-
-Here is a result of F<benchmark/x-poor-env.pl> to compare pure Perl template
-engines in poor environment where applications runs as CGI scripts and XS
-modules are not available.
-
-    $ perl -Mblib benchmark/x-poor-env.pl
-    Perl/5.10.1 i686-linux
-    Xslate backend: Booster
-    Text::Xslate/0.1058
-    Template/2.22
-    HTML::Template/2.9
-    Text::MicroTemplate/0.15
-    Text::MicroTemplate::Extended/0.11
-    1..3
-    ok 1 - TT: Template-Toolkit
-    ok 2 - MT: Text::MicroTemplate
-    ok 3 - HT: HTML::Template
-    Benchmarks with 'include' (datasize=100)
-             Rate     TT     HT Xslate     MT
-    TT     76.1/s     --   -60%   -61%   -82%
-    HT      189/s   149%     --    -3%   -56%
-    Xslate  196/s   158%     4%     --   -54%
-    MT      429/s   463%   126%   118%     --
-
-According to this result, PP::Booster is over 2 times faster than Template::Toolkit,
-and as fast as HTML::Template, but slower than Text::MicroTemplate.
-
 =head1 SEE ALSO
 
 L<Text::Xslate>
 
 L<Text::Xslate::PP::Opcode>
 
-L<Text::Xslate::PP::Booster>
-
 =head1 AUTHOR
 
 Text::Xslate::PP stuff is originally written by Makamaka Hannyaharamitu
