@@ -2,9 +2,12 @@ package Text::Xslate::PP::Opcode;
 use Any::Moose;
 extends qw(Text::Xslate::PP::State);
 
+our $VERSION = '1.0099_01';
+
 use Carp ();
 use Scalar::Util ();
 
+use Text::Xslate::PP;
 use Text::Xslate::PP::Const;
 use Text::Xslate::Util qw(
     p neat
@@ -16,7 +19,6 @@ use constant _DUMP_PP => scalar($DEBUG =~ /\b dump=pp \b/xms);
 
 no warnings 'recursion';
 
-require Text::Xslate::PP;
 if(!Text::Xslate::PP::_PP_ERROR_VERBOSE()) {
     our @CARP_NOT = qw(
         Text::Xslate
@@ -68,6 +70,25 @@ sub op_localize_s {
     goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
 }
 
+sub op_localize_vars {
+    my($st) = @_;
+    my $new_vars = $st->{sa};
+    my $old_vars = $st->vars;
+
+    if(ref($new_vars) ne 'HASH') {
+        $st->warn(undef, "Variable map must be a HASH reference");
+    }
+
+    push @{ $st->{local_stack} }, bless sub {
+            $st->vars($old_vars);
+            return;
+        }, 'Text::Xslate::PP::Guard';
+
+    $st->vars($new_vars);
+
+    goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
+}
+
 sub op_push {
     push @{ $_[0]->{ SP }->[ -1 ] }, $_[0]->{sa};
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
@@ -123,24 +144,7 @@ sub op_fetch_field_s {
 
 sub op_print {
     my($st) = @_;
-    my $sv = $st->{sa};
-
-    if ( ref( $sv ) eq TXt_RAW ) {
-        if(defined ${$sv}) {
-            $st->{ output } .= ${$sv};
-        }
-        else {
-            $st->warn(undef, "Use of nil to print" );
-        }
-    }
-    elsif ( defined $sv ) {
-        $sv =~ s/($Text::Xslate::PP::html_metachars)/$Text::Xslate::PP::html_escape{$1}/xmsgeo;
-        $st->{ output } .= $sv;
-    }
-    else {
-        $st->warn( undef, "Use of nil to print" );
-    }
-
+    $st->print($st->{sa});
     goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
 }
 
@@ -164,27 +168,19 @@ sub op_print_raw_s {
 
 
 sub op_include {
-    my $st = Text::Xslate::PP::tx_load_template( $_[0]->engine, $_[0]->{sa}, 1 );
-    $_[0]->{ output } .= Text::Xslate::PP::tx_execute( $st, $_[0]->{vars} );
-
-    goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
+    my($st) = @_;
+    my $child = Text::Xslate::PP::tx_load_template( $st->engine, $st->{sa}, 1 );
+    $st->push_frame('include', undef);
+    my $output = Text::Xslate::PP::tx_execute( $child, $st->{vars} );
+    $st->pop_frame(0);
+    $st->{output} .= $output;
+    goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
 }
 
-
 sub op_for_start {
     my($st) = @_;
-    my $ar = $st->{sa};
     my $id = $st->op_arg;
-
-    unless ( $ar and ref $ar eq 'ARRAY' ) {
-        if ( defined $ar ) {
-            $st->error( undef, "Iterator variables must be an ARRAY reference, not %s", neat( $ar ) );
-        }
-        else {
-            $st->warn( undef, "Use of nil to iterate" );
-        }
-        $ar = [];
-    }
+    my $ar = Text::Xslate::PP::tx_check_itr_ar($st, $st->{sa});
 
     #tx_access_lvar( $st, $id + TXfor_ITEM, undef );
     tx_access_lvar( $st, $id + TXfor_ITER, -1 );
@@ -208,6 +204,8 @@ sub op_for_iter {
             goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
         }
         else {
+            # finish the loop
+            $st->{sa} = ( $i > 0 ); # for 'for-else' block
             tx_access_lvar( $st, $id + TXfor_ITEM,  undef );
             tx_access_lvar( $st, $id + TXfor_ITER,  undef );
             tx_access_lvar( $st, $id + TXfor_ARRAY, undef );
@@ -374,12 +372,12 @@ sub op_builtin_uri {
 }
 
 sub op_builtin_is_array_ref {
-    $_[0]->{sa} = ref($_[0]->{sa}) eq 'ARRAY';
+    $_[0]->{sa} = Text::Xslate::Util::is_array_ref($_[0]->{sa});
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
 sub op_builtin_is_hash_ref {
-    $_[0]->{sa} = ref($_[0]->{sa}) eq 'HASH';
+    $_[0]->{sa} = Text::Xslate::Util::is_hash_ref($_[0]->{sa});
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
@@ -461,11 +459,9 @@ sub tx_macro_enter {
         return;
     }
 
-    my $cframe = Text::Xslate::PP::tx_push_frame( $st );
+    my $cframe = $st->push_frame($name, $retaddr);
 
-    $cframe->[ TXframe_RETADDR ] = $retaddr;
     $cframe->[ TXframe_OUTPUT ]  = $st->{ output };
-    $cframe->[ TXframe_NAME ]    = $name;
 
     $st->{ output } = '';
 
@@ -492,16 +488,14 @@ sub tx_macro_enter {
 
 sub op_macro_end {
     my($st) = @_;
-    my $frames   = $st->frame;
-    my $oldframe = $frames->[ $st->current_frame ];
-    my $cframe   = $frames->[ $st->current_frame( $st->current_frame - 1 ) ]; # pop frame
 
-    print STDERR " " x $st->current_frame, "tx_macro_end($oldframe->[ TXframe_NAME ])\n" if _DUMP_PP;
+    my $top = $st->frame->[ $st->current_frame ];
+    printf STDERR "%stx_macro_end(%s)]\n", ' ' x $st->current_frame - 1, $top->[ TXframe_NAME ] if _DUMP_PP;
 
     $st->{sa} = mark_raw( $st->{ output } );
+    $st->pop_frame(1);
 
-    $st->{ output } = $oldframe->[ TXframe_OUTPUT ];
-    $st->{ pc }     = $oldframe->[ TXframe_RETADDR ];
+    $st->{ pc } = $top->[ TXframe_RETADDR ];
     goto $st->{ code }->[ $st->{ pc } ]->{ exec_code };
 }
 
@@ -562,17 +556,17 @@ sub op_end {
     $st->{ pc } = $st->code_len;
 
     if($st->current_frame != 0) {
-        Carp::croak("Oops: broken stack frame:" .  p($st->frame));
+        #Carp::croak("Oops: broken stack frame:" .  p($st->frame));
     }
     return;
 }
 
-sub op_depend; *op_depend = \&op_noop;
-
+sub op_depend;      *op_depend      = \&op_noop;
 sub op_macro_begin; *op_macro_begin = \&op_noop;
 sub op_macro_nargs; *op_macro_nargs = \&op_noop;
 sub op_macro_outer; *op_macro_outer = \&op_noop;
 sub op_set_opinfo;  *op_set_opinfo  = \&op_noop;
+sub op_super;       *op_super       = \&op_noop;
 
 #
 # INTERNAL COMMON FUNCTIONS
