@@ -19,12 +19,11 @@ use constant _DUMP_TOKEN => scalar($DEBUG =~ /\b dump=token \b/xmsi);
 
 our @CARP_NOT = qw(Text::Xslate::Compiler Text::Xslate::Symbol);
 
-my $CODE    = qr/ (?: (?: $STRING | [^'"] )*? ) /xms; # ' for poor editors
-
+my $CODE    = qr/ (?: $STRING | [^'"] ) /xms;
 my $COMMENT = qr/\# [^\n;]* (?=[;\n])?/xms;
 
 # Operator tokens that the parser recognizes.
-# All the single characters are tokanized as an operator.
+# All the single characters are tokenized as an operator.
 my $OPERATOR_TOKEN = sprintf '(?:%s|[^ \t\r\n])', join('|', map{ quotemeta } qw(
     ...
     ..
@@ -208,7 +207,7 @@ sub parse {
 
     $parser->input( $parser->preprocess($input) );
 
-    $parser->next_token( $parser->tokanize() );
+    $parser->next_token( $parser->tokenize() );
     $parser->advance();
     my $ast = $parser->statements();
 
@@ -278,7 +277,7 @@ sub auto_chomp {
     return $nl;
 }
 
-# split templates by tags before tokanizing
+# split templates by tags before tokenizing
 sub split :method {
     my $parser  = shift;
     local($_) = @_;
@@ -291,7 +290,6 @@ sub split :method {
 
     my $lex_line_code = defined($line_start) && qr/\A ^ [ \t]* \Q$line_start\E ([^\n]* \n?) /xms;
     my $lex_tag_start = qr/\A \Q$tag_start\E ($CHOMP_FLAGS?)/xms;
-    my $lex_tag_end   = qr/\A ($CODE) ($CHOMP_FLAGS?) \Q$tag_end\E/xms;
 
     my $lex_text = qr/\A ( [^\n]*? (?: \n | (?= \Q$tag_start\E ) | \z ) ) /xms;
 
@@ -299,10 +297,25 @@ sub split :method {
 
     while($_) {
         if($in_tag) {
-            if(s/$lex_tag_end//xms) {
-                my($code, $chomp) = ($1, $2);
-                $in_tag = 0;
+            my $start = 0;
+            my $pos;
+            while( ($pos = index $_, $tag_end, $start) >= 0 ) {
+                my $code = substr $_, 0, $pos;
+                $code =~ s/$CODE//xmsog;
+                if(length($code) == 0) {
+                    last;
+                }
+                $start = $pos + 1;
+            }
 
+            if($pos >= 0) {
+                my $code = substr $_, 0, $pos, '';
+                $code =~ s/($CHOMP_FLAGS?) \z//xmso;
+                my $chomp = $1;
+
+                s/\A \Q$tag_end\E //xms or die "Oops!";
+
+                $in_tag = 0;
                 push @tokens, [ code => $code ];
                 if($chomp) {
                     push @tokens, [ postchomp => $chomp ];
@@ -434,8 +447,8 @@ sub _init_basic_symbols {
     $parser->symbol('=>') ->is_comma(1);
 
     # common commands
-    $parser->symbol('print')    ->set_std(\&std_command);
-    $parser->symbol('print_raw')->set_std(\&std_command);
+    $parser->symbol('print')    ->set_std(\&std_print);
+    $parser->symbol('print_raw')->set_std(\&std_print);
 
     # special literals
     $parser->define_literal(nil   => undef);
@@ -614,8 +627,8 @@ sub define_pair {
     return;
 }
 
-# the low-level tokanizer. Don't use it directly, use advance() instead.
-sub tokanize {
+# the low-level tokenizer. Don't use it directly, use advance() instead.
+sub tokenize {
     my($parser) = @_;
 
     local *_ = \$parser->{input};
@@ -652,7 +665,7 @@ sub next_token_is {
     return $parser->next_token->[1] eq $token;
 }
 
-# the high-level tokanizer
+# the high-level tokenizer
 sub advance {
     my($parser, $expect) = @_;
 
@@ -673,7 +686,7 @@ sub advance {
     $parser->statement_is_finished( $parser->following_newline != 0 );
     my $line = $parser->line( $parser->line + $parser->following_newline );
 
-    $parser->next_token( $parser->tokanize() );
+    $parser->next_token( $parser->tokenize() );
 
     my($arity, $id) = @{$t};
     if( $arity eq "name" && $parser->next_token_is("=>") ) {
@@ -1070,6 +1083,15 @@ sub define { # define a name to the scope
     return $symbol;
 }
 
+sub print {
+    my($parser, @args) = @_;
+    return $parser->symbol('print')->clone(
+        arity => 'print',
+        first => \@args,
+        line  => $parser->line,
+    );
+}
+
 sub binary {
     my($parser, $symbol, $lhs, $rhs) = @_;
     if(!ref $symbol) {
@@ -1099,7 +1121,9 @@ sub define_function {
     my($parser, @names) = @_;
 
     foreach my $name(@names) {
-        $parser->symbol($name)->set_nud(\&nud_name);
+        my $s = $parser->symbol($name);
+        $s->set_nud(\&nud_name);
+        $s->is_defined(1);
     }
     return;
 }
@@ -1154,11 +1178,7 @@ sub auto_command {
         return $expr;
     }
     else {
-        return $parser->symbol('print')->clone(
-            arity  => 'command',
-            first  => [$expr],
-            line   => $expr->line,
-        );
+        return $parser->print($expr);
     }
 }
 
@@ -1414,6 +1434,14 @@ sub std_for {
     my $proc = $symbol->clone(arity => 'for');
     $proc->first( $parser->expression(0) );
     $parser->pointy($proc, 1);
+    if($parser->token eq 'else') {
+        $parser->advance();
+        my $else = $parser->block();
+        $proc = $symbol->clone( arity => 'for_else',
+            first  => $proc,
+            second => $else,
+        )
+    }
     return $proc;
 }
 
@@ -1489,12 +1517,8 @@ sub std_macro_block {
             $call = $parser->call($filter, $call);
         }
     }
-    my $print = $parser->symbol('print')->clone(
-        arity => 'command',
-        first => [$call],
-    );
     # std() can return a list
-    return( $macro, $print );
+    return( $macro, $parser->print($call) );
 }
 
 sub std_override { # synonym to 'around'
@@ -1637,41 +1661,37 @@ sub build_given_body {
 sub std_include {
     my($parser, $symbol) = @_;
 
-    my $arg  = $parser->expression(0);
+    my $arg  = $parser->barename();
     my $vars = $parser->localize_vars();
     my $stmt = $symbol->clone(
-        first  => [$arg],
+        first  => $arg,
         second => $vars,
-        arity  => 'command',
+        arity  => 'include',
     );
     return $parser->finish_statement($stmt);
 }
 
-sub std_command {
+sub std_print {
     my($parser, $symbol) = @_;
     my $args;
     if($parser->token->id ne ";") {
         $args = $parser->expression_list();
     }
-    my $stmt = $symbol->clone(first => $args, arity => 'command');
+    my $stmt = $symbol->clone(first => $args, arity => 'print');
     return $parser->finish_statement($stmt);
 }
 
+# for cascade() and include()
 sub barename {
     my($parser) = @_;
 
     my $t = $parser->token;
-    if(!any_in($t->arity, qw(name literal))) {
-        $parser->_unexpected("a name or string literal", $t)
+    if($t->arity ne 'name' or $t->is_defined) {
+        # string literal for 'cascade', or any expression for 'include'
+        return $parser->expression(0);
     }
 
-    # "string" is ok
-    if($t->arity eq 'literal') {
-        $parser->advance();
-        return $t;
-    }
-
-    # package::name
+    # path::to::name
     my @parts;
     push @parts, $t;
     $parser->advance();
