@@ -4,6 +4,7 @@ use Any::Moose '::Util::TypeConstraints';
 
 use File::Spec     ();
 use File::Basename ();
+use Encode         ();
 
 with any_moose 'X::Getopt';
 
@@ -36,15 +37,27 @@ has module => (
     traits        => $getopt_traits,
 );
 
-has input_layer => (
-    documentation => 'Input layer for reading',
-    cmd_aliases   => [qw(l)],
-    is            => 'ro',
+has input_encoding => (
+    documentation => 'Input encoding (default: UTF-8)',
+    cmd_aliases   => [qw(ie)],
+    is            => 'rw',
     isa           => 'Str',
-    predicate     => 'has_input_layer',
+    default       => 'UTF-8',
+    predicate     => 'has_input_encoding',
     traits        => $getopt_traits,
 );
 
+has output_encoding => (
+    documentation => 'Output encoding (default: UTF-8)',
+    cmd_aliases   => [qw(oe)],
+    is            => 'rw',
+    isa           => 'Str',
+    default       => 'UTF-8',
+    predicate     => 'has_output_encoding',
+    traits        => $getopt_traits,
+);
+
+
 has path => (
     documentation => 'Include paths',
     cmd_aliases   => [qw(I)],
@@ -63,17 +76,17 @@ has syntax => (
     traits        => $getopt_traits,
 );
 
-has escape => (
-    documentation => 'Escaping mode (html or none)',
-    cmd_aliases   => [qw(p)],
+has type => (
+    documentation => 'Output content type (html | xml | text)',
+    cmd_aliases   => [qw(t)],
     is            => 'ro',
     isa           => 'Str',
-    predicate     => 'has_escape',
+    predicate     => 'has_type',
     traits        => $getopt_traits,
 );
 
 has verbose => (
-    documentation => 'Warning level (default to 2)',
+    documentation => 'Warning level (default: 2)',
     cmd_aliases   => [qw(w)],
     is            => 'ro',
     isa           => 'Str',
@@ -158,15 +171,13 @@ has version => (
 );
 
 sub run {
-    my $self = shift;
-
-    my $targets = $self->extra_argv;
+    my($self, @targets) = @_;
 
     my %args;
-    foreach my $field qw(
-        cache_dir cache input_layer path syntax
-        escape verbose
-            ) {
+    foreach my $field (qw(
+        cache_dir cache path syntax
+        type verbose
+            )) {
         my $method = "has_$field";
         $args{ $field } = $self->$field if $self->$method;
     }
@@ -179,6 +190,10 @@ sub run {
         $args{module} = \@mods;
     }
 
+    if(my $ie = $self->input_encoding) {
+        $args{input_layer} = ":encoding($ie)";
+    }
+
     local $ENV{XSLATE} = $self->debug
         if $self->has_debug;
 
@@ -197,12 +212,13 @@ sub run {
         if($self->has_define){
             %vars = %{$self->define};
         }
-        $vars{ARGV} = $targets;
+        $vars{ARGV} = \@targets;
+        $vars{ENV}  = \%ENV;
         print $xslate->render_string($self->eval, \%vars), "\n";
         return;
     }
 
-    foreach my $target (@$targets) {
+    foreach my $target (@targets) {
         # XXX if you have a directory, just pushed that into the list of
         # path in the xslate object
         if ( -d $target ) {
@@ -257,7 +273,7 @@ sub process_file {
 
     my $outfile;
 
-    if(defined $dest or not exists $suffix_map->{$suffix}) {
+    if(defined $dest or exists $suffix_map->{$suffix}) {
         $outfile= File::Spec->catfile( $dest, $filearg );
         if (my $replace = $suffix_map->{ $suffix }) {
             $outfile =~ s/$suffix$/$replace/;
@@ -272,9 +288,8 @@ sub process_file {
         }
     }
 
-
     my $rendered = $xslate->render( $filearg, $self->define );
-    utf8::is_utf8($rendered) && utf8::encode($rendered);
+    $rendered = Encode::encode($self->output_encoding, $rendered);
 
     if(defined $outfile) {
         my $fh;
@@ -315,9 +330,8 @@ This is the guts of C<xslate(1)>.
 
 =head1 AUTHOR
 
-Maki, Daisuke (lestrrat)
-
-Fuji, Goro (gfx)
+This is originally written by Maki, Daisuke (lestrrat),
+and also maintained by Fuji, Goro (gfx),
 
 =head1 SEE ALSO
 
